#include "stdafx.h"
#include "ModelReport.h"

const int STATERADIUS=10;
const COLORREF CR_STATEDARK = RGB(0,0,0);
const COLORREF CR_STATELIGHT = RGB(255,255,255);
const int ROWHEIGHT=12;

//-----------------------------------------------------------------------------	
CModelRptDisplay :: CModelRptDisplay( CModel * pModel ) 
	: CRptGroup( 50, 500, 570, 792, RGB(0,0,0) ) {

	CModelState * pState, *pState0, * pState1;
	int iDX, iDY, iAdjustX, iAdjustY;
	POINT pt0, pt1, ptA;
	int iRates=pModel->m_aRates.Count();
	int iStates=pModel->m_aStates.Count();

	//-- Outer Frame
	AddBox( m_iX0, m_iY0, m_iX1, m_iY1, true, false );
	
	//-- Connecting Rates
	for( int iRate=0; iRate<iRates; ++iRate ) { 
		CModelRate * pRate = pModel->Rate(iRate);
		//-- first state
		pState0 = pModel->State(pRate->m_aiStates[0]);
		pt0.x = X(pState0->m_dXPos);
		pt0.y = Y(pState0->m_dYPos);
		//-- second state
		pState1 = pModel->State(pRate->m_aiStates[1]);
		pt1.x = X(pState1->m_dXPos);
		pt1.y = Y(pState1->m_dYPos);
		//-- Draw rate if states are far enough apart.
		iDX = abs(pt1.x-pt0.x);
		iDY = abs(pt1.y-pt0.y);
		if( iDX > 2*STATERADIUS || iDY > 2*STATERADIUS ) { 
			//-- adjust x0, x1 to intersection points with state boxes					
			if( iDY > iDX ) { 
				iAdjustX = MulDiv(iDX,STATERADIUS,iDY) * (pt1.x>pt0.x?1:-1);
				iAdjustY = STATERADIUS * (pt1.y>pt0.y?1:-1);
				}
			else { 
				iAdjustX = STATERADIUS * (pt1.x>pt0.x?1:-1);
				iAdjustY = MulDiv(iDY,STATERADIUS,iDX) * (pt1.y>pt0.y?1:-1);
				}
			pt0.x += (iAdjustX+(iAdjustX>0?1:iAdjustX<0?-1:0));
			pt0.y += (iAdjustY+(iAdjustY>0?1:iAdjustY<0?-1:0));
			pt1.x -= iAdjustX;
			pt1.y -= iAdjustY;
			//-- Connect
			AddLine( pt0.x, pt0.y, pt1.x, pt1.y );
			//-- Arrow heads
			CalcArrowPoint( pt0, pt1, &ptA );
			AddLine( pt1.x, pt1.y, ptA.x, ptA.y );
			CalcArrowPoint( pt1, pt0, &ptA );
			AddLine( pt0.x, pt0.y, ptA.x, ptA.y );
			}
		}
		
	//-- States 
	char buf[16];
	for( int iState=0; iState<iStates; ++iState ) { 
		pState = pModel->State(iState);
		COLORREF cr = pModel->Class(pState->m_iClass)->m_cr;
		//-- boxes
		int iX = X(pState->m_dXPos);
		int iY = Y(pState->m_dYPos);
		AddBox( iX-STATERADIUS, iY-STATERADIUS, iX+STATERADIUS, iY+STATERADIUS, true, true, RGB(0,0,0), cr );
		//-- text labels. Color is black if sum of RGB components >= 384 
		sprintf( buf, "%d", iState );
		cr = ((cr&0xFF0000)>>16)+((cr&0xFF00)>>8)+((cr&0xFF))>=384 ? CR_STATEDARK : CR_STATELIGHT;
		AddText( 16, iX-STATERADIUS+(iState>=10?2:6), iY-STATERADIUS+3, buf, cr );
		}
		
	}

int CModelRptDisplay :: X( double dPct ) {
	return int( ((100.0+dPct)*m_iX1 + (100.0-dPct) * m_iX0 ) / 200.0 );
	}
	
int CModelRptDisplay :: Y( double dPct ){ 
	return int( ((100.0+dPct)*m_iY0 + (100.0-dPct) * m_iY1 ) / 200.0 );
	}
	
//-----------------------------------------------------------------------------	
CModelRpt :: CModelRpt( CModel * pModel ) { 
	
	CRptPage * pPage = AddPage();
	
	m_poDisp = new CModelRptDisplay( pModel );
	pPage->m_oGroup.m_aObjects.AddItem( m_poDisp );
	
	//pPage->AddBox( 370, 792-pModel->m_aRates.Count()*ROWHEIGHT, 600, 792, true, false, RGB(255,0,0), RGB(0,64,0) );
	
	}
